local L = AceLibrary("AceLocale-2.2"):new("GlowFoSho")
GlowFoSho = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceHook-2.1")
local dewdrop = AceLibrary("Dewdrop-2.0")
local currWeaponLink, currWeaponEnchant
GlowFoSho.eList = {}
local tbl = GlowFoSho.eList
local showGlowless = false
local onlyCompatible = true
--the button
local gfsbutton

--make enchant table local
local enchants = GlowFoSho_enchants

--[[

Local helper functions

--]]

--returns information about the enchant when provided with enchantID
local function enchantInfo(enchantID)
	if tonumber(enchantID) then
		for name, enchant in pairs(enchants) do
			if enchant.enchantID == enchantID then
				return name, enchantID, enchant.formulaID, enchant.glowless
			end
		end
		return
	end
end

--creates a link for an enchanting recipe with formulaID and name
local function GetRecipeLink(formulaID,name)
	local is2H = string.find(name," %(2H%)")
	if is2H then
		name = string.sub(name,1,is2H-1)
		is2H = "2H "
	else
		is2H = ""
	end
	
	return "|cffffd000|Henchant:" .. formulaID .. "|h[Enchant " .. is2H .. "Weapon - " .. name .. "]|h|r"
end

--sets enchant for item link to enchantID
local function SetEnchant(link, enchantID)
	local link1, link2 = string.match(link,"(.*item:%d+:)%d+(.*)")
	if link1 and link2 then
		return link1 .. enchantID .. link2
	end
	
	return
end

--returns the enchantID in the link
local function GetEnchant(link)
	return string.match(link,"item:%d+:(%d+)")
end

--returns the list of enchants for dewdrop menu
local function UpdateEnchantList()
	--if there is no weapon equipped, empty the list
	if not currWeaponLink then
		for k in pairs(tbl) do
			tbl[k] = nil
		end
		return
	end

	local weapMinlvl, _, _, _, is2H = select(5,GetItemInfo(currWeaponLink))
	is2H = (is2H == "INVTYPE_2HWEAPON")
	for name, enchant in pairs(enchants) do
		--get rid of glowless if those should not be shown
		if enchant.glowless and not showGlowless then
			tbl[name] = nil
		--remove incompatible enchants if those should not be shown
		elseif onlyCompatible and ((enchant.lvl and weapMinlvl < enchant.lvl) or (not is2H and string.find(name," %(2H%)"))) then
			tbl[name] = nil
		else
			tbl[name] = name
		end
	end
end

--[[

Main Addon Functions

--]]

function GlowFoSho:OnInitialize()
	--set up GlowFoSho button on the dressup frame
	gfsbutton = CreateFrame("Button","GlowFoSho_Button",UIParent)
	gfsbutton:Hide()
	--to be compatible with CloseUp, otherwise does not receive clicks
	gfsbutton:SetFrameStrata("HIGH")
	gfsbutton:SetWidth(25)
	gfsbutton:SetHeight(25)
	gfsbutton:SetNormalTexture("Interface\\Icons\\Ability_SearingArrow")
	gfsbutton:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	--set up button click scripts
	gfsbutton:RegisterForClicks("LeftButtonUp","RightButtonUp")
	gfsbutton:SetScript("OnClick",function() if dewdrop:IsOpen(GlowFoSho_Button) then dewdrop:Close() else dewdrop:Open(GlowFoSho_Button) end end)
end

function GlowFoSho:OnEnable()
	--attach button to DressUpFrame, so it shows up when opened
	gfsbutton:SetParent(DressUpFrame)
	gfsbutton:SetPoint("TOPRIGHT","DressUpFrame","TOPRIGHT",-50,-80)
	gfsbutton:Show()

	--create the dropdown menu
	dewdrop:Register(GlowFoSho_Button,
					"point","TOPLEFT",
					"relativePoint","TOPRIGHT",
					"children",{
						type = "group",
						args = {
							getWeaponLink = {
								name = L["Show Weapon Link"],
								desc = L["Displays the link of the enchanted weapon in the chat frame."],
								type = "execute",
								func = "GetWeaponLink",
								handler = self,
								order = 10
							},
							getFormulaLink = {
								name = L["Show Enchant Link"],
								desc = L["Displays the link of the enchant currently on the weapon in the chat frame. CAUTION: links are not secure and may diconnect you!"],
								type = "execute",
								func = "GetEnchantLink",
								handler = self,
								order = 20
							},
							glowless = {
								name = L["Show Glowless Enchants"],
								desc = L["Allows you to preview enchants which do not add glow to a weapon."],
								type = "toggle",
								get = function() return showGlowless end,
								set = function(v) showGlowless = v; UpdateEnchantList() end,
								order = 30
							},
							compatible = {
								name = L["Show Only Compatible Enchants"],
								desc = L["Filters out enchants that cannot be applied to the currently previewed weapon."],
								type = "toggle",
								get = function() return onlyCompatible end,
								set = function(v) onlyCompatible = v; UpdateEnchantList() end,
								order = 35
							},
							enchants = {
								name = L["Enchants"],
								desc = L["List of weapon enchants you can preview."],
								type = "text",
								get = function() return currWeaponEnchant end,
								set = "EnchantWeaponByName",
								validate = self.eList,
								handler = self,
								order = 40
							},
						}
					},
					"dontHook", true)

	--hook onto DressUp frame functions
	self:SecureHook("DressUpItemLink")
	self:HookScript(DressUpFrame,"OnShow","OnDressUpFrameShow")
	self:HookScript(DressUpFrameResetButton,"OnClick","OnDressUpFrameResetButtonClick")
end

function GlowFoSho:OnDisable()
	--hide the button
	gfsbutton:ClearAllPoints()
	gfsbutton:SetParent(UIParent)
	gfsbutton:Hide()
end

--hook to DressUpItemLink function
function GlowFoSho:DressUpItemLink(link)
	if link then
		local iType
		_, link, _, _, _, iType = GetItemInfo(link)

		--if previewing weapon, set the currWeaponLink link to passed link
		if iType == "Weapon" then
			currWeaponLink = link
		end
	end

	if not currWeaponLink then
		currWeaponLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	end

	if currWeaponLink then
		currWeaponEnchant = enchantInfo(GetEnchant(currWeaponLink))
	end
	UpdateEnchantList()
end

--reset previous enchants when DressUp frame shows up
function GlowFoSho:OnDressUpFrameShow(...)
	currWeaponLink = nil
	currWeaponEnchant = nil
	self.hooks[DressUpFrame]["OnShow"](...)
end

--reset previous enchants when Reset button is pressed.
--enchant is set to currently equiped weapon data
function GlowFoSho:OnDressUpFrameResetButtonClick(...)
	currWeaponLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	if currWeaponLink then
		currWeaponEnchant = enchantInfo(GetEnchant(currWeaponLink))
	end
	self.hooks[DressUpFrameResetButton]["OnClick"](...)
end

--enchant and preview weapon
-- args: enchant name as it appears in the enchants table
function GlowFoSho:EnchantWeaponByName(name)
	if currWeaponLink and enchants[name] then
		self:EnchantWeapon(enchants[name].enchantID)
		currWeaponEnchant = name
	end
end

--enchant and preview weapon by enchantID
function GlowFoSho:EnchantWeapon(enchantID)
	if currWeaponLink then
		local newLink = SetEnchant(currWeaponLink,enchantID)
		if newLink then
			currWeaponLink = newLink
			DressUpItemLink(currWeaponLink)
		end
	end
end

--prints out link to the currently previewed weapon with a currently previewed enchant
function GlowFoSho:GetWeaponLink()
	if currWeaponLink then
		self:Print(currWeaponLink)
	end
end

--prints out link to a formula for the current enchant on the weapon
function GlowFoSho:GetEnchantLink()
	local name, _, formulaID = self:GetCurrEnchant()
	if name then
		self:Print(GetRecipeLink(formulaID,name))
	else
		self:Print(L["There is no enchant on the weapon or enchant unknown."])
	end
end

--returns information about the enchant on currently preview weapon
function GlowFoSho:GetCurrEnchant()
	local enchant = enchants[currWeaponEnchant]
	if enchant then
		return currWeaponEnchant, enchant.enchantID, enchant.formulaID, enchant.glowless
	end
end
